from turbogears import controllers, expose, flash
import model
from turbogears import identity, redirect, url
from cherrypy import request, response, config
# from tgcrashdb import json
#import logging
#log = logging.getLogger("tgcrashdb.controllers")
from turbogears import widgets, validators
import os
from apport.report import Report

crashdir = config.get("crashdb.crashdir",os.path.join(os.getcwd(),"crashdb"))
coredir = os.path.join(crashdir,"core")

class UUIDField(widgets.WidgetsList):
    """A simple field for requesting a UUID to look up"""
    uuid = widgets.TextField(label="UUID",
                             validator=validators.NotEmpty(),
                             attrs=dict(size="36"))

class UploadField(widgets.WidgetsList):
    """A file upload field for apport Reports"""
    data = widgets.FileField(label="Crash Report",
                             validator=validators.NotEmpty())

def writecrash(infile,uuid):
    '''Write the file object given by infile out to a crashfile'''
    crashfile = os.path.join(crashdir,"%s.crash" % uuid)
    if not os.path.exists(crashdir):
        os.mkdir(crashdir)
    outfile = open(crashfile,"w")
    while True:
        data = infile.read(8192)
        if not data:
            break
        outfile.write(data)
    outfile.close()
    return crashfile

class CrashController(controllers.Controller):
    @expose()
    def index(self):
        raise redirect(url('/'))

    def getkey(self,crash,key):
        if not hasattr(crash,key):
            # FIXME uhhhh how do we do this The Right Way?
            #cherrypy.NotFound(self)
            return 'HELLA 404 YO'
        else:
            return str(getattr(crash,key))

    @expose(template=".templates.showcrash")
    def default(self,uuid,key=None):
        '''Show a crash given by uuid'''
        crash = None
        s = model.CrashReport.selectBy(uuid=uuid)
        if s.count() == 0:
            flash('UUID "%s" not found' % uuid)
        else:
            crash = s.getOne()

        # Update the object if the crashfile has been modified
        crashfile = os.path.join(crashdir,crash.uuid + '.crash')
        # ugly way to force a datetime.datetime object to a timestamp..
        if int(crash.timestamp.strftime("%s")) < os.stat(crashfile)[8]:
            crash.read(open(crashfile),False)
            crash.timestamp = crash.timestamp.now()
        
        # Check to see if a particular key was requested
        if key == "CoreDump":
            r = Report()
            r.load(open(crashfile))
            # FIXME set proper content-type
            return r['CoreDump']
        elif key:
            return self.getkey(crash,key)
        else:
            return dict(uuid=uuid, crash=crash)


class Root(controllers.RootController):
    crash = CrashController()

    @expose(template=".templates.welcome")
    def index(self):
        uuid_form = widgets.TableForm(fields=UUIDField(),action="lookup",submit_text="Show Crash")
        return dict(uuid_form=uuid_form)

    @expose(template=".templates.browsecrashes")
    @identity.require(identity.in_group("admin"))
    def browsecrashes(self):
        '''Browse the crash database.'''
        crashes = model.CrashReport.select()
        return dict(crashes=crashes)

    @expose()
    def lookup(self, uuid):
        '''fake-lookup a UUID. Just redirect to the right place.'''
        raise redirect(url('/crash/%s' % uuid))

    @expose(template=".templates.upload")
    def upload(self,data=None):
        if data is not None:
            # receiving a file by POST
            c = model.CrashReport() # gives us c.uuid
            # write crash file
            crashfile = writecrash(data.file,c.uuid)
            # FIXME write core dump so developer can download it?
            c.read(open(crashfile),False) # skip the CoreDump and other binary bits
            # show user their new crashdb entry
            self.lookup(c.uuid)
        else:
            upload_form = widgets.TableForm(fields=UploadField(),action="upload",submit_text="Upload")
            return dict(upload_form=upload_form)

    @expose(template=".templates.login")
    def login(self, forward_url=None, previous_url=None, *args, **kw):

        if not identity.current.anonymous \
            and identity.was_login_attempted() \
            and not identity.get_identity_errors():
            raise redirect(forward_url)

        forward_url=None
        previous_url= request.path

        if identity.was_login_attempted():
            msg=_("The credentials you supplied were not correct or "
                   "did not grant access to this resource.")
        elif identity.get_identity_errors():
            msg=_("You must provide your credentials before accessing "
                   "this resource.")
        else:
            msg=_("Please log in.")
            forward_url= request.headers.get("Referer", "/")
            
        response.status=403
        return dict(message=msg, previous_url=previous_url, logging_in=True,
                    original_parameters=request.params,
                    forward_url=forward_url)

    @expose()
    def logout(self):
        identity.current.logout()
        raise redirect("/")
